import { act, render, screen } from '@testing-library/react'
import userEvent from '@testing-library/user-event'
import Header from 'components/Header/Header'
import { ScreenType } from 'enums'
import React from 'react'
import { Provider } from 'react-redux'
import * as ModalAction from 'store/modal/modal.slice'
import { createMockEcolyoStore, mockGlobalState } from 'tests/__mocks__/store'

const mockedNavigate = jest.fn()
jest.mock('react-router-dom', () => ({
  ...jest.requireActual('react-router-dom'),
  useNavigate: () => mockedNavigate,
}))

describe('Header component', () => {
  const mobileStore = createMockEcolyoStore()
  const desktopStore = createMockEcolyoStore({
    global: { ...mockGlobalState, screenType: ScreenType.DESKTOP },
  })

  beforeEach(() => {
    jest.clearAllMocks()
  })

  it('should be rendered correctly on mobile', () => {
    const { container } = render(
      <Provider store={mobileStore}>
        <Header desktopTitleKey="mockKey" />
      </Provider>
    )
    expect(container).toMatchSnapshot()
    expect(container.getElementsByTagName('header')[0]).toBeInTheDocument()
  })

  it('should be rendered correctly on desktop', () => {
    const { container } = render(
      <Provider store={desktopStore}>
        <Header desktopTitleKey="mockKey" />
      </Provider>
    )
    expect(container).toMatchSnapshot()
    expect(container.getElementsByTagName('header')[0]).toBeInTheDocument()
  })

  it('should display desktop title text when desktopTitleKey provided', () => {
    const titleKey = 'Test Title'
    render(
      <Provider store={desktopStore}>
        <Header desktopTitleKey={titleKey} />
      </Provider>
    )
    expect(screen.getByText(titleKey)).toBeInTheDocument()
  })

  it('should not display desktop title text when desktopTitleKey not provided', () => {
    const { queryByText } = render(
      <Provider store={desktopStore}>
        <Header desktopTitleKey="" />
      </Provider>
    )
    expect(queryByText(/.+/)).not.toBeInTheDocument()
  })

  it('should display title and back button when desktopTitle key provided and displayBackArrow is true', () => {
    const key = 'titleKey'
    render(
      <Provider store={desktopStore}>
        <Header desktopTitleKey={key} displayBackArrow={true} />
      </Provider>
    )
    expect(screen.getByText(key)).toBeInTheDocument()
    expect(
      screen.getByLabelText('header.accessibility.button_back')
    ).toBeInTheDocument()
  })

  it('should NOT display back button when displayBackArrow is false', () => {
    render(
      <Provider store={mobileStore}>
        <Header desktopTitleKey="test" displayBackArrow={false} />
      </Provider>
    )
    expect(
      screen.queryByRole('button', {
        name: 'header.accessibility.button_back',
      })
    ).not.toBeInTheDocument()
  })

  describe('back functions', () => {
    it('should call navigate -1 when back button is clicked with NO back fn', async () => {
      render(
        <Provider store={desktopStore}>
          <Header desktopTitleKey="KEY" displayBackArrow={true} />
        </Provider>
      )
      await act(async () => {
        await userEvent.click(
          screen.getByLabelText('header.accessibility.button_back')
        )
      })
      expect(mockedNavigate).toHaveBeenCalled()
    })

    it('should call custom back fn when back button is clicked', async () => {
      const mockBack = jest.fn()
      render(
        <Provider store={desktopStore}>
          <Header
            desktopTitleKey="KEY"
            displayBackArrow={true}
            backFunction={mockBack}
          />
        </Provider>
      )
      await act(async () => {
        await userEvent.click(
          screen.getByLabelText('header.accessibility.button_back')
        )
      })
      expect(mockBack).toHaveBeenCalled()
    })
  })

  it('should call handleClickFeedbacks when feedback button is clicked', async () => {
    render(
      <Provider store={mobileStore}>
        <Header desktopTitleKey="mockKey" />
      </Provider>
    )
    const updateModalSpy = jest.spyOn(ModalAction, 'openFeedbackModal')
    await act(async () => {
      await userEvent.click(
        screen.getByLabelText('header.accessibility.button_open_feedbacks')
      )
    })
    expect(updateModalSpy).toHaveBeenCalledWith(true)
  })
})
