import * as Sentry from '@sentry/react'
import { Client, Q, QueryResult } from 'cozy-client'
import logger from 'cozy-logger'
import { FLUIDSPRICES_DOCTYPE } from 'doctypes'
import { FluidType } from 'enums'
import { DateTime } from 'luxon'
import { FluidPrice } from 'models'
import logApp from 'utils/logger'
import ConfigService from './fluidConfig.service'

const logStack = logger.namespace('fluidPricesService')

export default class FluidPricesService {
  private readonly _client: Client

  constructor(_client: Client) {
    this._client = _client
  }

  /**
   * Get a price according to a fluidType and a data. This method return the nearest and valid price for the given date.
   */
  public async getPrices(
    fluidType: FluidType,
    date: DateTime
  ): Promise<FluidPrice> {
    const query = Q(FLUIDSPRICES_DOCTYPE)
      .where({
        startDate: {
          $lte: date.toISO({ suppressMilliseconds: true }).toString(),
        },
        fluidType,
      })
      .indexFields(['startDate'])
      .sortBy([{ startDate: 'desc' }])
      .limitBy(1)

    const { data: fluidsPrices }: QueryResult<FluidPrice[]> =
      await this._client.query(query)
    return fluidsPrices[0]
  }

  /**
   * Get current prices for all fluidTypes.
   */
  public async getAllLastPrices(): Promise<FluidPrice[]> {
    const query = Q(FLUIDSPRICES_DOCTYPE)
      .where({ endDate: { $eq: '' } })
      .indexFields(['fluidType'])
      .sortBy([{ fluidType: 'asc' }])
      .limitBy(3)

    const { data: fluidsPrices }: QueryResult<FluidPrice[]> =
      await this._client.query(query)

    // If some data is missing, recover it using default config
    if (fluidsPrices.length !== 3) {
      const configService = new ConfigService()
      const fluidConfig = configService.getFluidConfig()
      const recoveredFluidsPrices: FluidPrice[] = []

      // Getting default fluid price
      fluidConfig.forEach(config => {
        recoveredFluidsPrices.push({
          _id: '0',
          fluidType: config.fluidTypeId,
          price: config.coefficient,
          startDate: config.startDate,
          endDate: '',
          UpdatedAt: '2000-01-01T00:00:00Z',
        })
      })

      // Replacing default prices if available
      fluidsPrices.forEach(fluidPrice => {
        recoveredFluidsPrices[fluidPrice.fluidType] = fluidPrice
      })

      return recoveredFluidsPrices
    }

    return fluidsPrices
  }

  /**
   * Check if a fluidPrice exists in db
   * @returns {Promise<FluidPrice | null>} price or null
   */
  public async checkIfPriceExists(
    fluidPrice: FluidPrice
  ): Promise<FluidPrice | null> {
    const query = Q(FLUIDSPRICES_DOCTYPE).where({
      startDate: { $eq: fluidPrice.startDate },
      fluidType: { $eq: fluidPrice.fluidType },
    })
    const {
      data: [price],
    }: QueryResult<FluidPrice[]> = await this._client.query(query)
    if (price) return price
    return null
  }

  /**
   * Creates a new fluidPrice
   * @returns  {Promise<FluidPrice | null>}  price or null
   */
  public async createPrice(newPrice: FluidPrice): Promise<FluidPrice | null> {
    try {
      const { data: createdPrice }: QueryResult<FluidPrice> =
        await this._client.create(FLUIDSPRICES_DOCTYPE, newPrice)
      return createdPrice
    } catch (error) {
      const errorMessage = `'Error creating new createdPrice: ${JSON.stringify(
        error
      )}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(error)
      throw error
    }
  }

  /**
   * Updates a price in db
   */
  public async updatePrice(
    doc: FluidPrice,
    attributes: Partial<FluidPrice>
  ): Promise<FluidPrice | null> {
    const { data: fluidPrice }: QueryResult<FluidPrice | null> =
      await this._client.save({
        ...doc,
        ...attributes,
      })
    if (fluidPrice) return fluidPrice
    return null
  }
}
