import * as Sentry from '@sentry/react'
import { Client, Q, QueryResult } from 'cozy-client'
import triggersMutations from 'cozy-harvest-lib/dist/connections/triggers'
import { buildAttributes } from 'cozy-harvest-lib/dist/helpers/triggers'
import logger from 'cozy-logger'
import { TRIGGERS_DOCTYPE } from 'doctypes'
import {
  Account,
  Konnector,
  Trigger,
  TriggerAttributes,
  TriggerState,
} from 'models'
import ConfigService from 'services/fluidConfig.service'
import logApp from 'utils/logger'

const logStack = logger.namespace('triggersService')

export default class TriggerService {
  private _client: Client

  constructor(_client: Client) {
    this._client = _client
  }

  private createTriggerAttributes(account: Account, konnector: Konnector) {
    const configService = new ConfigService()
    const cronArgs = configService.getCronArgs(7, 9)
    const triggerAttributes: TriggerAttributes = buildAttributes({
      account: account,
      cron: cronArgs,
      folder: null,
      konnector: konnector,
    })
    return triggerAttributes
  }

  public async createTrigger(
    account: Account,
    konnector: Konnector
  ): Promise<Trigger> {
    const triggerAttributes = this.createTriggerAttributes(account, konnector)
    const trigger = await triggersMutations(this._client).createTrigger(
      triggerAttributes
    )
    return trigger
  }

  public async getTrigger(
    account: Account,
    konnector: Konnector
  ): Promise<Trigger | null> {
    const query = Q(TRIGGERS_DOCTYPE)
      .where({
        'message.account': account._id,
        'message.konnector': konnector.slug,
      })
      .indexFields(['cozyMetadata.updatedAt'])
      .sortBy([{ 'cozyMetadata.updatedAt': 'desc' }])
      .limitBy(1)
    const { data: triggers }: QueryResult<Trigger[]> =
      await this._client.query(query)
    return triggers[0] ? triggers[0] : null
  }

  public async getTriggerForAccount(account: Account): Promise<Trigger | null> {
    const query = Q(TRIGGERS_DOCTYPE)
      .where({
        'message.account': account._id,
        'message.konnector': account.account_type,
      })
      .indexFields(['cozyMetadata.createdAt'])
      .sortBy([{ 'cozyMetadata.createdAt': 'desc' }])
      .limitBy(1)
    const { data: triggers }: QueryResult<Trigger[]> =
      await this._client.query(query)
    return triggers[0] ? triggers[0] : null
  }

  public async fetchTriggerState(
    trigger: Trigger
  ): Promise<TriggerState | null> {
    try {
      const triggerState = await this._client
        .getStackClient()
        .fetchJSON('GET', `/jobs/triggers/${trigger._id}`)
      // eslint-disable-next-line camelcase
      return triggerState.data.attributes.current_state
        ? triggerState.data.attributes.current_state
        : null
    } catch (error) {
      throw new Error('Fetch trigger state failed')
    }
  }

  public async deleteTrigger(trigger: Trigger): Promise<boolean> {
    try {
      await this._client.destroy(trigger)
      return true
    } catch (error) {
      const errorMessage = `Delete trigger failed: ${JSON.stringify(error)}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(error)
      throw new Error('Delete trigger failed')
    }
  }
}
