import { Client } from 'cozy-client'
import logger from 'cozy-logger'
import {
  REMOTE_ORG_ECOLYO_AGENT_PRICES,
  REMOTE_ORG_ECOLYO_AGENT_PRICES_REC,
} from 'doctypes'
import { FluidType } from 'enums'
import { FluidPrice } from 'models'
import EnvironmentService from 'services/environment.service'
import FluidPricesService from 'services/fluidsPrices.service'
import { runService } from './service'

const logStack = logger.namespace('fluidPrices')

const getRemotePricesByFluid = async (
  client: Client,
  fluidType: FluidType
): Promise<FluidPrice[]> => {
  const env = new EnvironmentService()
  const remoteUrl = env.isProduction()
    ? REMOTE_ORG_ECOLYO_AGENT_PRICES
    : REMOTE_ORG_ECOLYO_AGENT_PRICES_REC
  const prices = await client
    .getStackClient()
    .fetchJSON('GET', `${remoteUrl}?fluidtype=${fluidType}`)
  return prices
}

const updatePrices = async ({ client }: { client: Client }) => {
  for (const fluidType of [
    FluidType.ELECTRICITY,
    FluidType.WATER,
    FluidType.GAS,
  ]) {
    logStack('info', `Updating fluid prices for ${fluidType}...`)
    const fps = new FluidPricesService(client)
    const remotePrices = await getRemotePricesByFluid(client, fluidType)
    for (const remotePrice of remotePrices) {
      const existingPrice = await fps.checkIfPriceExists(remotePrice)
      if (!existingPrice) {
        logStack('debug', `Price doesn't exist in db, creating a new price`)
        await fps.createPrice(remotePrice)
        continue
      }

      if (existingPrice.UpdatedAt >= remotePrice.UpdatedAt) {
        logStack('debug', `Price up to date`)
        continue
      }
      logStack('debug', `Price exists in db but not up to date, updating it`)
      // If a price has been updated, set the oldest startDate of the edited price so we can redo aggregation

      await fps.updatePrice(existingPrice, {
        price: remotePrice.price,
        UpdatedAt: remotePrice.UpdatedAt,
        startDate: remotePrice.startDate,
        endDate: remotePrice.endDate,
      })
      logStack('info', `Price updated`)
    }
  }
}

runService(updatePrices)
